@@ -1,6 +1,6 @@
-use chrono::Utc;
 use serde::{Deserialize, Serialize};
 use std::borrow::Cow;
+use time::OffsetDateTime;
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct UserResponse {
@@ -36,7 +36,8 @@ pub struct LoginResponse {
 #[derive(Debug, Serialize, Deserialize)]
 pub struct AddHistoryRequest {
     pub id: String,
-    pub timestamp: chrono::DateTime<Utc>,
+    #[serde(with = "time::serde::rfc3339")]
+    pub timestamp: OffsetDateTime,
     pub data: String,
     pub hostname: String,
 }
@@ -48,8 +49,10 @@ pub struct CountResponse {
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct SyncHistoryRequest {
-    pub sync_ts: chrono::DateTime<chrono::FixedOffset>,
-    pub history_ts: chrono::DateTime<chrono::FixedOffset>,
+    #[serde(with = "time::serde::rfc3339")]
+    pub sync_ts: OffsetDateTime,
+    #[serde(with = "time::serde::rfc3339")]
+    pub history_ts: OffsetDateTime,
     pub host: String,
 }
 
