@@ -12,13 +12,14 @@ mod sync;
 #[cfg(feature = "sync")]
 mod account;
 
+mod config;
 mod history;
 mod import;
 mod kv;
 mod search;
 mod stats;
 
-#[derive(Subcommand)]
+#[derive(Subcommand, Debug)]
 #[command(infer_subcommands = true)]
 pub enum Cmd {
     /// Manipulate shell history
@@ -44,6 +45,10 @@ pub enum Cmd {
 
     #[command(subcommand)]
     Kv(kv::Cmd),
+
+    /// Print example configuration
+    #[command()]
+    DefaultConfig,
 }
 
 impl Cmd {
@@ -54,27 +59,34 @@ impl Cmd {
             .parse_env("ATUIN_LOG")
             .init();
 
+        tracing::trace!(command = ?self, "client command");
+
         let mut settings = Settings::new().wrap_err("could not load client settings")?;
 
         let db_path = PathBuf::from(settings.db_path.as_str());
         let record_store_path = PathBuf::from(settings.record_store_path.as_str());
 
-        let mut db = Sqlite::new(db_path).await?;
+        let db = Sqlite::new(db_path).await?;
         let mut store = SqliteStore::new(record_store_path).await?;
 
         match self {
-            Self::History(history) => history.run(&settings, &mut db).await,
-            Self::Import(import) => import.run(&mut db).await,
-            Self::Stats(stats) => stats.run(&mut db, &settings).await,
+            Self::History(history) => history.run(&settings, &db).await,
+            Self::Import(import) => import.run(&db).await,
+            Self::Stats(stats) => stats.run(&db, &settings).await,
             Self::Search(search) => search.run(db, &mut settings).await,
 
             #[cfg(feature = "sync")]
-            Self::Sync(sync) => sync.run(settings, &mut db, &mut store).await,
+            Self::Sync(sync) => sync.run(settings, &db, &mut store).await,
 
             #[cfg(feature = "sync")]
             Self::Account(account) => account.run(settings).await,
 
             Self::Kv(kv) => kv.run(&settings, &mut store).await,
+
+            Self::DefaultConfig => {
+                config::run();
+                Ok(())
+            }
         }
     }
 }
