@@ -35,7 +35,7 @@ pub trait SearchEngine: Send + Sync + 'static {
     async fn query(&mut self, state: &SearchState, db: &mut dyn Database) -> Result<Vec<History>> {
         if state.input.as_str().is_empty() {
             Ok(db
-                .list(state.filter_mode, &state.context, Some(200), true)
+                .list(state.filter_mode, &state.context, Some(200), true, false)
                 .await?
                 .into_iter()
                 .collect::<Vec<_>>())
