@@ -7,10 +7,8 @@ use eyre::Result;
 use command::AtuinCmd;
 mod command;
 
-#[allow(clippy::all)]
-mod ratatui;
-
 const VERSION: &str = env!("CARGO_PKG_VERSION");
+const SHA: &str = env!("GIT_HASH");
 
 static HELP_TEMPLATE: &str = "\
 {before-help}{name} {version}
